using System;
using System.IO;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions; 
namespace OnyxBox.Utility
{
	/// <summary>
	/// Abstraction of an INI File. Allows easy reading and writing of an INIFile.
	/// </summary>
	public class IniFile
	{
		private const int FIELD_IX = 0;
		private const int VALUE_IX = 1;

		private string _fullPath;
		private string _section;
		
		// default constuctor, path to the file name must be supplied
		public IniFile(string path)
		{
			this.Path = path;
			this.Section = "";
		}

		// the full path to the INI file.
		public string Path
		{
			get
			{
				return (_fullPath);
			}
			set
			{
				if(!File.Exists(value))
				{
					FileStream newFile = File.Create(value);
					newFile.Close();
				}
				
				// CreateText above may raise an exception so we set the
				// full pull path member var here.
				_fullPath = value;
			}
		}

		// the current section within the INI file.
		public string Section
		{
			get
			{
				return(_section);
			}
			set
			{
				if (Regex.IsMatch(value,"\\[|\\]"))
				{
					throw(new ArgumentException("Invalid char used in section name."));
				}
				_section = value;
			}
		}

		// returns an array containing all of the section names within the INI file.
		public string[] GetSectionNames()
		{
			ArrayList sections = new ArrayList();
			StreamReader iniFile = File.OpenText(_fullPath);
			
			string line;

			while ((line = iniFile.ReadLine()) != null) 
			{
				if (line.StartsWith("["))
				{
					string sectionName = line.Substring (1, line.LastIndexOf("]") - 1);
					sections.Add (sectionName);
				}
			}

			iniFile.Close();
			return ((string[])sections.ToArray(typeof(String)));
		}
		
		// returns an array containing all the field names in the current section
		public string[] GetFieldNames()
		{
			ArrayList fields = new ArrayList();
			StreamReader iniFile = File.OpenText(this.Path);

			string line = "";
			bool foundSection = false;
	
			// now read all of the fields within the section
			while ((line = iniFile.ReadLine()) != null &! foundSection)
			{
				// is the current line the section def.
				foundSection = (Regex.IsMatch(line, "^\\[" + this.Section + "\\]", RegexOptions.IgnoreCase));
				
				if (foundSection)
				{
					// found the section, now read the fields.
					while((line = iniFile.ReadLine()) != null)
					{
						line = line.Trim();
						if (line.StartsWith("["))
						{
							break;
						}
						else if (!(line.StartsWith(";") || line.Length == 0 || line.StartsWith("#")))
						{
							string field = line.Split('=')[0];
							fields.Add (field);
						}
					}
				}
			}

			iniFile.Close();
			return ((string[])fields.ToArray(typeof(String)));
		}

		public string[] GetFieldNames(string section)
		{
			this.Section = section;
			return(this.GetFieldNames());
		}

		// write / alter the value of a field within the current section.
		public bool SetValue(string field, string val)
		{
			bool valueHasBeenSet = false;
			
			// get the existing INI file and read it all
			FileStream iniFs = new FileStream(this.Path, FileMode.Open);
			StreamReader oldIni = new StreamReader(iniFs);			
			string[] lines = oldIni.ReadToEnd().Split('\n');
			
			// define the new line that will be placed into the INI file
			field = field.Trim();	
			string newLine = (val.Length > 0) ? field + "=" + val  : "";
			
			ArrayList newIniContent = new ArrayList();
			
			bool sectionFound = false;
			bool reachedLastLine = false;
			for (int index = 0; index < lines.Length; index++)
			{
				string line = lines[index];
				string lineToAppend = line;

				if (! valueHasBeenSet)
				{
					if (Regex.IsMatch(line, "\\[" + this.Section + "\\]", RegexOptions.IgnoreCase))
					{
						sectionFound = true;
					}
					else if(sectionFound && line.StartsWith(field + "="))
					{
						lineToAppend = newLine;
						valueHasBeenSet = true;
					}
					else if(sectionFound && line.StartsWith("["))
					{
						newIniContent.Add (newLine);
						valueHasBeenSet = true;
					}
					else if(sectionFound && reachedLastLine)
					{
						newIniContent.Add (line);
						lineToAppend = newLine;
					}
				
				}

				if(lineToAppend.Length > 0)  // this class filters out any blank lines
				{
					newIniContent.Add (lineToAppend);
				}

			}

			if (!sectionFound)
			{
				// append a new section and field / value to the file
				if (this.Section.Length > 0)
				{
					newIniContent.Add ("[" + this.Section + "]");
					newIniContent.Add (newLine);
					valueHasBeenSet = true;
				}
				else
				{	
					// just stick the new Field / value above all the other content
					newIniContent.Insert(0, newLine);
					valueHasBeenSet = true;	
				}
			}
			else if(!valueHasBeenSet)
			{
				newIniContent.Add (newLine);
				valueHasBeenSet = true;
			}

			try
			{
				// remove the contents of the iniFile
				iniFs.SetLength(0); 
				// write the newIniContent string into the file
				StreamWriter newIni = new StreamWriter(iniFs);
				string output = String.Join("\n", (string[])newIniContent.ToArray(typeof(string)));
				newIni.Write(output);
				newIni.Close();
			}
			finally
			{
				iniFs.Close();	
			}

			return (valueHasBeenSet);
		}

		public bool SetValue(string section, string field, string val)
		{
			this.Section = section;
			return(this.SetValue(field,val));
		}
		
		// get a field's value from the current section
		public string GetValue(string field)
		{	
			string retVal = null;
			StreamReader iniFile = File.OpenText(this.Path);

			string line;
			bool foundSection = false;
	
			// now read all of the fields within the section
			while ((line = iniFile.ReadLine()) != null &! foundSection)
			{
				line = line.Trim();
				// is the current line the section def.
				foundSection = (Regex.IsMatch(line, "\\[" + this.Section + "\\]", RegexOptions.IgnoreCase));
				
				if (foundSection)
				{
					// found the section, now read the fields.
					while((line = iniFile.ReadLine()) != null)
					{
						line = line.Trim();
						if  (line.StartsWith("["))
						{
							break;
						}
						else if(!( line.StartsWith(";") || line.Length == 0 || line.StartsWith("#")))
						{
							string[] fieldAndValue = line.Split('=');
							if (fieldAndValue[FIELD_IX] == field)
							{
								retVal = fieldAndValue[VALUE_IX];
								break;
							}						
						}
					}
				}
			}
			
			iniFile.Close();
			return(retVal);
		}

		public string GetValue(string section, string field)
		{
			this.Section = section;
			return(this.GetValue(field));
		}

		// deletes a field from the current section
		public bool DeleteField(string field)
		{
			return(this.SetValue(field,""));
		}

		public bool DeleteField(string section,string field)
		{
			this.Section = section;
			return(this.SetValue(field,""));
		}

		// deletes a section from the file
		public bool DeleteSection()
		{	
			bool retVal = false;
			
			// get the existing INI file and read it all
			FileStream iniFs = new FileStream(this.Path, FileMode.Open);
			StreamReader oldIni = new StreamReader(iniFs);			
			string[] lines = oldIni.ReadToEnd().Split('\n');

			ArrayList newIniContent = new ArrayList();			
			bool includeLine = true;
			for (int index = 0; index < lines.Length; index++)
			{
				string line = lines[index];
				if (Regex.IsMatch(line, "\\[" + this.Section + "\\]", RegexOptions.IgnoreCase))
				{
					includeLine = false;
					retVal = true;
				}
				else if(!includeLine && line.StartsWith("["))
				{
					includeLine = true;
				}
			
				if(includeLine)
				{
					newIniContent.Add (line);
				}
			
			}

			try
			{
				iniFs.SetLength(0); 
				// write the newIniContent string into the file
				StreamWriter newIni = new StreamWriter(iniFs);
				string output = String.Join("\n", (string[])newIniContent.ToArray(typeof(string)));
				newIni.Write(output);
				newIni.Close();
			}
			finally
			{				
				iniFs.Close();	
			}

			return (retVal);

		}
		
		public bool DeleteSection(string section)
		{
			this.Section = section;
			return(this.DeleteSection());
		}

	}









}